@@ -15,10 +15,13 @@ from binance.client import Client
 import numpy as np
 from napoleontoolbox.connector import napoleon_s3_connector
 import pandas as pd
+
 from datetime import datetime, timedelta
 
 import hashlib
 
+from napoleontoolbox.connector import napoleon_connector
+
 def pair_handling_message(fetcher, msg, compute_minute_signal=True, compute_day_signal=True, compute_parallel = False):
     high_dict = fetcher.pair_dict['high']
     low_dict = fetcher.pair_dict['low']
@@ -206,7 +209,7 @@ def pair_handling_message(fetcher, msg, compute_minute_signal=True, compute_day_
     volu_dict[trade_time_unix_timestamp_minute] = actual_volu + float(msg['q'])
 
 class RealTimeSignalEnsemblingParalleLauncher():
-    def __init__(self, min_calib_starting_date=None, min_calib_running_date=None, day_calib_starting_date=None, day_calib_running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, min_calib_selected_algo='', day_calib_selected_algo='', user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl'):
+    def __init__(self, min_calib_starting_date=None, min_calib_running_date=None, day_calib_starting_date=None, day_calib_running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, min_calib_selected_algo='', day_calib_selected_algo='', user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl', backed_up_signal_histo_suffix='backed_up_signal_histo.pkl', load_daily_histo = True):
         self.min_calib_starting_date = min_calib_starting_date
         self.min_calib_running_date = min_calib_running_date
 
@@ -281,6 +284,9 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print('days_selected algos')
         print(self.day_calib_list_pkl_file_name)
 
+        self.backed_up_signal_histo_suffix = backed_up_signal_histo_suffix
+
+
         if self.dropbox_backup:
             self.min_calib_signals_list =  self.dbx.download_pkl(self.min_calib_list_pkl_file_name)
             self.day_calib_signals_list =  self.dbx.download_pkl(self.day_calib_list_pkl_file_name)
@@ -306,6 +312,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
 
         self.day_signal_mapping={}
         day_max_lookback_window = 0
+        self.day_lookback_windows = []
         for me_signal in self.day_calib_signals_list:
             run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
             params = json.loads(run_json_string)
@@ -313,6 +320,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
             params = json.loads(run_json_string)
             readable_label = params['signal_type'] + str(params['trigger']) + salty
             self.day_signal_mapping[me_signal]=readable_label
+            self.day_lookback_windows.append(params['lookback_window'])
             if params['lookback_window']>day_max_lookback_window:
                 day_max_lookback_window=params['lookback_window']
         self.day_max_lookback_window = day_max_lookback_window
@@ -327,6 +335,9 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
         self.last_minute = None
         self.last_day = None
+        self.load_daily_histo = load_daily_histo
+        if self.load_daily_histo:
+            self.reload_daily_histo()
 
 
     def get_aws_last_past_minutes(self, nb_last_hours = 24):
@@ -424,6 +435,148 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(aggregated_df.shape)
         return aggregated_df
 
+    def get_aws_last_past_days(self, nb_last_days = 50):
+        date_now = datetime.utcnow()
+        files_to_fetch = []
+
+        d = date_now
+        for i in range(nb_last_days):
+            d = d - timedelta(days=1)
+            day_name = self.pair + '_' + d.strftime('%d_%b_%Y') + '_dayohlc'
+            files_to_fetch.append(day_name)
+        aggregated_df = None
+        for me_file in files_to_fetch:
+            df = None
+            try:
+                df = self.aws_client.download_dataframe_from_csv(self.bucket, me_file)
+            except Exception as e:
+                print(e)
+                df = None
+            if aggregated_df is not None:
+                if df is not None:
+                    print(me_file)
+                    print(df.shape)
+                    aggregated_df = pd.concat([aggregated_df, df])
+            if aggregated_df is None:
+                if df is not None:
+                    print(df.shape)
+                    aggregated_df = df
+
+        print('size fetched')
+        print(aggregated_df.shape)
+
+        aggregated_df = aggregated_df.sort_values(by=['date'])
+        aggregated_df['date'] = pd.to_datetime(aggregated_df['date'])
+        aggregated_df = aggregated_df.set_index(aggregated_df['date'])
+        aggregated_df = aggregated_df.drop(columns=['date'])
+        test = aggregated_df.pivot_table(index=['date'], aggfunc='size')
+        print('max duplication')
+        print(test.max())
+
+        aggregated_df = aggregated_df.drop_duplicates()
+        test = aggregated_df.pivot_table(index=['date'], aggfunc='size')
+        print('max duplication  after dropping duplicates')
+        print(test.max())
+
+        print('size fetched after dropping duplicates')
+        print(aggregated_df.shape)
+        return aggregated_df
+
+    def recompute_daily_histo(self, from_crypto_compare = False):
+        if from_crypto_compare:
+            print('recomputing historical up to '+str(self.day_max_lookback_window))
+            running_date = datetime.now()
+            starting_date = running_date - timedelta(days=2*self.day_max_lookback_window)
+            daily_df = napoleon_connector.fetch_crypto_daily_data(ssj=self.underlying, local_root_directory=self.local_root_directory,
+                                                                   daily_return_pkl_filename_suffix='_daily_temp.pkl',
+                                                                   refetch_all=True,
+                                                                   daily_crypto_starting_day=starting_date)
+            daily_df = daily_df.sort_index()
+            print('time range for daily recomputation')
+            print(max(daily_df.index))
+            print(min(daily_df.index))
+            print(daily_df.shape)
+            print('data refetched')
+            self.recreate_daily_histo(daily_df,running_date)
+        else :
+            print('not enough ohlc binance history')
+
+
+    def recreate_daily_histo(self, daily_df, up_to_date):
+        cumulated_signals = None
+        print('Number of signals to aggregate '+str(len(self.day_calib_signals_list)))
+        skipping_size = self.day_max_lookback_window
+        mapping_dictionary = {}
+        counter = 1
+        for me_signal in self.day_calib_signals_list:
+            print('backtesting '+str(counter)+' over '+str(len(self.day_calib_signals_list)))
+            # we have to reload the file each time
+            freqly_df = daily_df.copy()
+            ## idiosyncratic run itself
+            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
+            params = json.loads(run_json_string)
+
+
+            normalization = params['normalization']
+            transaction_costs = params['transaction_costs']
+            trigger = params['trigger']
+
+            signal_type = params['signal_type']
+            if normalization and not signal_generator.is_signal_continuum(signal_type):
+                return
+
+            params = json.loads(run_json_string)
+            salty = str(int(hashlib.sha1(run_json_string.encode('utf-8')).hexdigest(), 16) % (10 ** 8))
+            params = json.loads(run_json_string)
+            signal_column = params['signal_type'] + str(params['trigger']) + salty
+
+            mapping_dictionary[me_signal]=signal_column
+
+            if signal_type == 'long_only':
+                freqly_df['signal']=1.
+            else:
+                lookback_window = params['lookback_window']
+                if skipping_size < lookback_window:
+                    raise Exception('The skipping size must be greater than the lookback window')
+                # kwargs = {**generics, **idios}
+                signal_generation_method_to_call = getattr(signal_generator, signal_type)
+                freqly_df = signal_utility.roll_wrapper(freqly_df, lookback_window, skipping_size,
+                                          lambda x: signal_generation_method_to_call(data=x, **params), trigger)
+                freqly_df, _ = signal_utility.reconstitute_signal_perf(data = freqly_df, transaction_cost = transaction_costs, normalization = normalization)
+
+            if cumulated_signals is None:
+                cumulated_signals = freqly_df[['close','high','low','open','volumefrom','signal']].copy()
+                cumulated_signals = cumulated_signals.rename(columns={'signal': signal_column})
+            else :
+                cumulated_signals[signal_column]=freqly_df['signal'].copy()
+            counter=counter+1
+
+        print('recomputation done')
+        print(cumulated_signals.shape)
+        cumulated_signals['date'] = pd.to_datetime(cumulated_signals.index)
+        cumulated_signals['ts'] = cumulated_signals['date'].values.astype(np.int64) // 10 ** 9
+
+        backed_up_signal_histo_name =  up_to_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_signal_histo_suffix
+        saving_path = self.local_root_directory+backed_up_signal_histo_name
+        print('saving histo recomputation to '+backed_up_signal_histo_name)
+        cumulated_signals.to_pickle(saving_path)
+
+    def reload_daily_histo(self):
+        upto_date = datetime.now()
+        backed_up_signal_histo_name =  upto_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_signal_histo_suffix
+        saving_path = self.local_root_directory+backed_up_signal_histo_name
+        print('loading histo recomputation to '+backed_up_signal_histo_name)
+        updated_df = pd.read_pickle(saving_path)
+        updated_df=updated_df.sort_index()
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        me_columns = self.generic_columns + signal_columns
+        updated_df = updated_df[me_columns].copy()
+        day_df = updated_df[self.generic_columns].copy()
+        self.day_signal_np = updated_df.values
+        self.day_np = day_df.values
+        self.day_df = day_df
+
+
     def start_fetching_pair(self):
         bm = BinanceSocketManager(self.client)
         pair_callback = partial(pair_handling_message, self)
@@ -485,14 +638,23 @@ class RealTimeSignalEnsemblingParalleLauncher():
         # updated_df = updated_df.astype(
         #     {'ts': 'float64', 'open': 'float64', 'high': 'float64', 'low': 'float64', 'close': 'float64','volumefrom': 'float64'}
         #                                )
-        updated_df['ts']=updated_df['ts'].astype(int)
-        updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
-        updated_df = updated_df.astype({'hour_ts': 'int64'})
-        updated_df = updated_df.astype({'day_ts': 'int64'})
-        if updated_df['day_ts'].nunique()>1:
+        # updated_df['ts']=updated_df['ts'].astype(int)
+        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
+        # updated_df = updated_df.astype({'hour_ts': 'int64'})
+        # updated_df = updated_df.astype({'day_ts': 'int64'})
+        # if updated_df['day_ts'].nunique()>1:
+        #     print('too many days saved together : investigate')
+        # day_timestamp = updated_df['day_ts'].max()
+
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        me_ts =updated_df['ts']
+        me_day_ts = me_ts-me_ts%(3600*24)
+        me_day_ts = me_day_ts.astype({'ts': 'int64'})
+        if me_day_ts.nunique()>1:
             print('too many days saved together : investigate')
-        day_timestamp = updated_df['day_ts'].max()
+        day_timestamp = me_day_ts.max()
+
         day_dt_object = datetime.utcfromtimestamp(day_timestamp)
         filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y') #str(hourr_timestamp)
         updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
@@ -511,12 +673,21 @@ class RealTimeSignalEnsemblingParalleLauncher():
         signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
         me_columns = self.generic_columns + signal_columns
         updated_df=pd.DataFrame(data=self.minute_signal_np, columns = me_columns, index=range(len(self.minute_signal_np)))
-        updated_df['ts']=updated_df['ts'].astype(int)
-        updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        updated_df = updated_df.astype({'hour_ts': 'int64'})
-        if updated_df['hour_ts'].nunique()>1:
-            print('too many hours saved together : investigate')
-        hour_timestamp = updated_df['hour_ts'].max()
+        #updated_df['ts']=updated_df['ts'].astype(int)
+        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        # updated_df = updated_df.astype({'hour_ts': 'int64'})
+        # if updated_df['hour_ts'].nunique()>1:
+        #     print('too many hours saved together : investigate')
+        # hour_timestamp = updated_df['hour_ts'].max()
+
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        me_ts =updated_df['ts']
+        me_hour_ts = me_ts-me_ts%3600
+        me_hour_ts = me_hour_ts.astype({'ts': 'int64'})
+        if me_hour_ts.nunique()>1:
+            print('too many days saved together : investigate')
+        hour_timestamp = me_hour_ts.max()
+
         hour_dt_object = datetime.utcfromtimestamp(hour_timestamp)
         filename = self.pair+'_'+hour_dt_object.strftime('%d_%b_%Y_%H') #str(hourr_timestamp)
         updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
@@ -535,15 +706,23 @@ class RealTimeSignalEnsemblingParalleLauncher():
         signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
         me_columns = self.generic_columns + signal_columns
         updated_df=pd.DataFrame(data=self.day_signal_np, columns = me_columns, index=range(len(self.day_signal_np)))
-        updated_df['ts']=updated_df['ts'].astype(int)
-        updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
-        updated_df = updated_df.astype({'day_ts': 'int64'})
-        if updated_df['day_ts'].nunique()>1:
+        # updated_df['ts']=updated_df['ts'].astype(int)
+        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
+        # updated_df = updated_df.astype({'day_ts': 'int64'})
+        # if updated_df['day_ts'].nunique()>1:
+        #     print('too many days saved together : investigate')
+        # day_timestamp = updated_df['day_ts'].max()
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        me_ts =updated_df['ts']
+        me_day_ts = me_ts-me_ts%(3600*24)
+        me_day_ts = me_day_ts.astype({'ts': 'int64'})
+        if me_day_ts.nunique()>1:
             print('too many days saved together : investigate')
-        day_timestamp = updated_df['day_ts'].max()
+        day_timestamp = me_day_ts.max()
+
         day_dt_object = datetime.utcfromtimestamp(day_timestamp)
-        filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y') #str(hourr_timestamp)
+        filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y')+'_dayohlc' #str(hourr_timestamp)
         updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
         updated_df = updated_df.sort_values(by=['ts'])
         updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
@@ -561,16 +740,30 @@ class RealTimeSignalEnsemblingParalleLauncher():
 
     def launchDayParallelPool(self, use_num_cpu):
         self.update_day_dataframe()
-        print('launching parallel computation for alphas at '+str(self.last_day))
+        print('launching parallel daily computation for alphas at '+str(self.last_day))
         with Pool(processes=use_num_cpu) as pool:
             run_results = pool.starmap(self.runDaySignal, self.day_calib_signals_list)
         print('parallel computation done')
         print('results length')
         print(len(run_results))
+        new_data_sig = np.array(run_results)
+        if len(self.day_np.shape)>1:
+            new_data_sig = np.hstack((self.day_np[-1,:].reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+        else:
+            new_data_sig = np.hstack((self.day_np.reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+        print('new daily data signal')
+        print(new_data_sig)
+        if self.day_signal_np is None:
+            self.day_signal_np = new_data_sig
+        else:
+            self.day_signal_np = np.vstack((self.day_signal_np, new_data_sig))
+        if len(self.day_signal_np.shape)>1 and len(self.day_signal_np)>3:
+            print('last three daily signals')
+            print(self.day_signal_np[-3:])
 
     def launchDaySequential(self):
         self.update_day_dataframe()
-        print('launching sequential computation for alphas at '+str(self.last_day))
+        print('launching sequential daily computation for alphas at '+str(self.last_day))
         run_results = []
         for meArg in self.day_calib_signals_list:
             run_results.append(self.runDaySignal(meArg))
@@ -600,6 +793,20 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print('parallel computation done')
         print('results length')
         print(len(run_results))
+        new_data_sig = np.array(run_results)
+        if len(self.minute_np.shape)>1:
+            new_data_sig = np.hstack((self.minute_np[-1,:].reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+        else:
+            new_data_sig = np.hstack((self.minute_np.reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+
+        print(new_data_sig)
+        if self.minute_signal_np is None:
+            self.minute_signal_np = new_data_sig
+        else:
+            self.minute_signal_np = np.vstack((self.minute_signal_np, new_data_sig))
+        if len(self.minute_signal_np.shape)>1 and len(self.minute_signal_np)>3:
+            print('last three signals')
+            print(self.minute_signal_np[-3:])
 
     def launchMinuteSequential(self):
         self.update_minute_dataframe()
@@ -655,13 +862,19 @@ class RealTimeSignalEnsemblingParalleLauncher():
         if normalization and not signal_generator.is_signal_continuum(signal_type):
             return
         lookback_window = params['lookback_window']
-        if lookback_window>self.minute_df.shape[0]:
+        if lookback_window>self.day_df.shape[0]:
             return np.nan
         freqly = self.day_df.iloc[-lookback_window:,:].copy()
         signal_generation_method_to_call = getattr(signal_generator, signal_type)
+
+        #last_generated_signal = np.nan
+        #try:
         last_generated_signal = signal_utility.compute_last_signal(freqly, lookback_window,
-                                                                   lambda x: signal_generation_method_to_call(
-                                                                       data=x, **params))
+                                                                       lambda x: signal_generation_method_to_call(
+                                                                           data=x, **params))
+        #except Exception as e:
+        #    print('Trouble computing signal ', e)
+
         return last_generated_signal
 
 
