@@ -65,6 +65,13 @@ def recover_to_sql_column_format(run):
     run = run.replace('dodo',':')
     return run
 
+
+def compute_last_signal(rolled_df, lookback_window, function_to_apply):
+    last_rolling_df = rolled_df.iloc[-lookback_window:]
+    last_rolling_bis_df = rolled_df.tail(lookback_window)
+    last_signal = function_to_apply(last_rolling_df)
+    return last_signal
+
 def roll_wrapper(rolled_df, lookback_window, skipping_size, function_to_apply, trigger):
     signal_df = roll(rolled_df, lookback_window).apply(function_to_apply)
     signal_df = signal_df.to_frame()
